package be.iminds.ilabt.jfed.experiment.setup.config;

import com.fasterxml.jackson.annotation.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class GenerateUsingJFedRequestRSpec extends RequestRSpec {
    private final int nodeCount;
    @Nullable
    private final String jFedResourceClass; //if null, a class will be picked automatically, using the order in the jFed GUI (top being highest priority)
    @Nonnull
    private final List<String> nodeAnsibleGroup;

    public GenerateUsingJFedRequestRSpec() {
        super(RequestRSpecSource.GENERATE_USING_JFED, false, false);
        this.nodeCount = 1;
        this.jFedResourceClass = null;
        this.nodeAnsibleGroup = Collections.emptyList();
    }

    @JsonCreator
    public GenerateUsingJFedRequestRSpec(@Nonnull @JsonProperty("source") RequestRSpecSource source,
                                         @Nullable @JsonProperty("nodeCount") Integer nodeCount,
                                         @Nullable @JsonProperty("jFedResourceClass") String jFedResourceClass,
                                         @Nullable @JsonProperty("nodeAnsibleGroup") List<String> nodeAnsibleGroup,
                                         @Nullable @JsonProperty("autoAssignSpecificNodesUsingListResources") Boolean autoAssignSpecificNodesUsingListResources,
                                         @Nullable @JsonProperty("useNitosLease") Boolean useNitosLease) {
        super(RequestRSpecSource.GENERATE_USING_JFED, autoAssignSpecificNodesUsingListResources, useNitosLease);
        assert source == RequestRSpecSource.GENERATE_USING_JFED;
        assert this.getSource() == RequestRSpecSource.GENERATE_USING_JFED;
        this.nodeCount = nodeCount == null ? new GenerateUsingJFedRequestRSpec().getNodeCount() : nodeCount;
        this.jFedResourceClass = jFedResourceClass == null ? new GenerateUsingJFedRequestRSpec().getjFedResourceClass() : jFedResourceClass;
        this.nodeAnsibleGroup = nodeAnsibleGroup == null ? new GenerateSimpleRequestRSpec().getNodeAnsibleGroup() : nodeAnsibleGroup;
    }

    @JsonIgnore
    public boolean isValid() {
        if (nodeCount <= 0)
            throw new IllegalArgumentException("Config Error: rspec.nodeCount == "+nodeCount+" makes no sense if rspec.source == "+source);
        return true;
    }

    @JsonProperty
    public int getNodeCount() {
        return nodeCount;
    }

    @JsonProperty
    @Nullable
    public String getjFedResourceClass() {
        return jFedResourceClass;
    }

    @JsonProperty
    @Nonnull
    public List<String> getNodeAnsibleGroup() {
        return nodeAnsibleGroup;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof GenerateUsingJFedRequestRSpec)) return false;
        if (!super.equals(o)) return false;

        GenerateUsingJFedRequestRSpec that = (GenerateUsingJFedRequestRSpec) o;

        if (nodeCount != that.nodeCount) return false;
        if (!nodeAnsibleGroup.equals(that.nodeAnsibleGroup)) return false;
        return jFedResourceClass != null ? jFedResourceClass.equals(that.jFedResourceClass) : that.jFedResourceClass == null;
    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        result = 31 * result + nodeCount;
        result = 31 * result + (jFedResourceClass != null ? jFedResourceClass.hashCode() : 0);
        result = 31 * result + nodeAnsibleGroup.hashCode();
        return result;
    }
}
